@@ -280,12 +280,30 @@ public:
 	{
 		slct_main = port;
 		slct_port = port;
+#ifdef WIRE_COMPRESS_SUPPORT
+		slct_zport = NULL;
+#endif
 	}
 
 	// get port to check for readyness
 	// assume port_mutex is locked
 	HandleState checkNext(RemPortPtr& port)
 	{
+#ifdef WIRE_COMPRESS_SUPPORT
+		if (slct_zport)
+		{
+			if ((slct_zport->port_flags & PORT_z_data) &&
+				(slct_zport->port_state != rem_port::DISCONNECTED))
+			{
+				port = slct_zport;
+				slct_zport = NULL;	// Will be set again by select_multi() if needed
+				return SEL_READY;
+			}
+
+			slct_zport = NULL;
+		}
+#endif
+
 		if (slct_port && slct_port->port_state == rem_port::DISCONNECTED)
 		{
 			// restart from main port
@@ -309,6 +327,13 @@ public:
 		return ok(port);
 	}
 
+	void setZDataPort(RemPortPtr& port)
+	{
+#ifdef WIRE_COMPRESS_SUPPORT
+		slct_zport = port;
+#endif
+	}
+
 	HandleState ok(const rem_port* port)
 	{
 #ifdef WIRE_COMPRESS_SUPPORT
@@ -317,7 +342,13 @@ public:
 #endif
 		SOCKET n = port->port_handle;
 #if defined(WIN_NT)
-		return FD_ISSET(n, &slct_fdset) ? SEL_READY : SEL_NO_DATA;
+		if (FD_ISSET(n, &slct_fdset))
+		{
+			unset(n);
+			return SEL_READY;
+		}
+		else
+			return SEL_NO_DATA;
 #elif defined(HAVE_POLL)
 		pollfd* pf = NULL;
 		FB_SIZE_T pos;
@@ -334,7 +365,14 @@ public:
 #else
 		if (n < 0 || n >= FD_SETSIZE)
 			return port->port_flags & PORT_disconnect ? SEL_DISCONNECTED : SEL_BAD;
-		return (n < slct_width && FD_ISSET(n, &slct_fdset)) ? SEL_READY : SEL_NO_DATA;
+
+		if (n < slct_width && FD_ISSET(n, &slct_fdset))
+		{
+			unset(n);
+			return SEL_READY;
+		}
+		else
+			return SEL_NO_DATA;
 #endif
 	}
 
@@ -388,6 +426,9 @@ public:
 #endif
 		slct_main = NULL;
 		slct_port = NULL;
+#ifdef WIRE_COMPRESS_SUPPORT
+		slct_zport = NULL;
+#endif
 	}
 
 	void select(timeval* timeout)
@@ -456,6 +497,9 @@ private:
 #endif
 	RemPortPtr slct_main;	// first port to check for readyness
 	RemPortPtr slct_port;	// next port to check for readyness
+#ifdef WIRE_COMPRESS_SUPPORT
+	RemPortPtr slct_zport;	// port with some compressed data remaining in the buffer
+#endif
 };
 
 static bool		accept_connection(rem_port*, const P_CNCT*);
@@ -499,9 +543,9 @@ static SocketsArray* forkSockets;
 static void		get_peer_info(rem_port*);
 
 static void		inet_gen_error(bool, rem_port*, const Arg::StatusVector& v);
-static bool_t	inet_getbytes(XDR*, SCHAR *, u_int);
+static bool_t	inet_getbytes(XDR*, SCHAR *, unsigned);
 static void		inet_error(bool, rem_port*, const TEXT*, ISC_STATUS, int);
-static bool_t	inet_putbytes(XDR*, const SCHAR*, u_int);
+static bool_t	inet_putbytes(XDR*, const SCHAR*, unsigned);
 static bool		inet_read(XDR*);
 static rem_port*		inet_try_connect(	PACKET*,
 									Rdb*,
@@ -1780,7 +1824,7 @@ static void disconnect(rem_port* const port)
 		SOCLOSE(port->port_channel);
 	}
 
-	if (port->port_thread_guard && port->port_events_thread && !Thread::isCurrent(port->port_events_threadId))
+	if (port->port_thread_guard && port->port_events_thread && !port->port_events_threadId.isCurrent())
 		port->port_thread_guard->setWait(port->port_events_thread);
 	else
 		port->releasePort();
@@ -2085,6 +2129,10 @@ static bool select_multi(rem_port* main_port, UCHAR* buffer, SSHORT bufsize, SSH
 				{
 					*length = 0;
 				}
+#ifdef WIRE_COMPRESS_SUPPORT
+				if (port->port_flags & PORT_z_data)
+					INET_select->setZDataPort(port);
+#endif
 				return (*length) ? true : false;
 			}
 
@@ -2109,6 +2157,10 @@ static bool select_multi(rem_port* main_port, UCHAR* buffer, SSHORT bufsize, SSH
 				}
 				*length = 0;
 			}
+#ifdef WIRE_COMPRESS_SUPPORT
+			if (port->port_flags & PORT_z_data)
+				INET_select->setZDataPort(port);
+#endif
 			return (*length) ? true : false;
 		}
 		if (!select_wait(main_port, &INET_select))
@@ -2543,7 +2595,7 @@ static void inet_gen_error(bool releasePort, rem_port* port, const Arg::StatusVe
 }
 
 
-static bool_t inet_getbytes( XDR* xdrs, SCHAR* buff, u_int count)
+static bool_t inet_getbytes( XDR* xdrs, SCHAR* buff, unsigned bytecount)
 {
 /**************************************
  *
@@ -2557,15 +2609,11 @@ static bool_t inet_getbytes( XDR* xdrs, SCHAR* buff, u_int count)
  **************************************/
 	const rem_port* port = (rem_port*) xdrs->x_public;
 	if (port->port_flags & PORT_server)
-	{
-		return REMOTE_getbytes(xdrs, buff, count);
-	}
-
-	SLONG bytecount = count;
+		return REMOTE_getbytes(xdrs, buff, bytecount);
 
 	// Use memcpy to optimize bulk transfers.
 
-	while (bytecount > (SLONG) sizeof(ISC_QUAD))
+	while (bytecount > sizeof(ISC_QUAD))
 	{
 		if (xdrs->x_handy >= bytecount)
 		{
@@ -2603,9 +2651,9 @@ static bool_t inet_getbytes( XDR* xdrs, SCHAR* buff, u_int count)
 		return TRUE;
 	}
 
-	while (--bytecount >= 0)
+	while (bytecount--)
 	{
-		if (!xdrs->x_handy && !inet_read(xdrs))
+		if (xdrs->x_handy == 0 && !inet_read(xdrs))
 			return FALSE;
 		*buff++ = *xdrs->x_private++;
 		--xdrs->x_handy;
@@ -2677,7 +2725,7 @@ static void inet_error(bool releasePort, rem_port* port, const TEXT* function, I
 	}
 }
 
-static bool_t inet_putbytes( XDR* xdrs, const SCHAR* buff, u_int count)
+static bool_t inet_putbytes( XDR* xdrs, const SCHAR* buff, unsigned bytecount)
 {
 /**************************************
  *
@@ -2689,11 +2737,10 @@ static bool_t inet_putbytes( XDR* xdrs, const SCHAR* buff, u_int count)
  *	Put a bunch of bytes to a memory stream if it fits.
  *
  **************************************/
-	SLONG bytecount = count;
 
 	// Use memcpy to optimize bulk transfers.
 
-	while (bytecount > (SLONG) sizeof(ISC_QUAD))
+	while (bytecount > sizeof(ISC_QUAD))
 	{
 		if (xdrs->x_handy >= bytecount)
 		{
@@ -2733,9 +2780,9 @@ static bool_t inet_putbytes( XDR* xdrs, const SCHAR* buff, u_int count)
 		return TRUE;
 	}
 
-	while (--bytecount >= 0)
+	while (bytecount--)
 	{
-		if (xdrs->x_handy <= 0 && !REMOTE_deflate(xdrs, inet_write, packet_send, false))
+		if (xdrs->x_handy == 0 && !REMOTE_deflate(xdrs, inet_write, packet_send, false))
 			return FALSE;
 		--xdrs->x_handy;
 		*xdrs->x_private++ = *buff++;
@@ -2777,7 +2824,7 @@ static bool inet_read( XDR* xdrs)
 		return false;
 	p += length;
 
-	xdrs->x_handy = (int) ((SCHAR *) p - xdrs->x_base);
+	xdrs->x_handy = (SCHAR *) p - xdrs->x_base;
 	xdrs->x_private = xdrs->x_base;
 
 	return true;
